(***********************************************************************)
(*                                                                     *)
(*                               Grimage                               *)
(*                                                                     *)
(*                           Graphics image                            *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2014-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: pixmap.mli,v 1.11 2014/02/27 16:45:22 fclement Exp $ *)

(** Pixmap: extra graphics functions primitives: handling pixels and
  pixmaps, using HSB and RGB models of colors. *)

open Generic_image;;

type saturation_threshold = int and brightness_threshold = int;;
 (** The staturation and brightness thresholds for deciding if a pixel
   has to be turned out to white or black during the mapping to bitmap of a
   color image. *)

val draw_pixmap : pixmap -> x -> y -> unit;;
 (** [draw_pixmap pixmap x y] draw the given color matrix into the
   graphics window with lower left corner at the given point. *)

val bitmap_of_pixmap :
  saturation_threshold -> brightness_threshold -> pixmap -> pixmap
;;
 (** [bitmap_of_pixmap ts tb pixmap] Maps a matrix of colors into a bitmap
   represented as a new matrix of black or white colors. Arguments [ts] and
   [tb] are tresholds to decide whether to produce a black or white pixel from
   a pixel of a given color: both [ts] and [tb] are integers in the range [0
   .. 255]; [ts] is the saturation threshold and if the saturation of the pixel
   color is greater than [ts] then the pixel is assumed to be [white]; [tb] is
   the brightness threshold and if the brightness of the pixel color is greater
   than [ts] then the pixel is assumed to be [white]. The correct values of
   [ts] and [tb] highly depend on the image at hand, but [ts = 20] and [tb =
   210] often give good results. *)

val bluemap_of_red_channel : pixmap -> bluemap;;
val bluemap_of_green_channel : pixmap -> bluemap;;
val bluemap_of_blue_channel : pixmap -> bluemap;;
 (** [bluemap_of_red_channel pxm] returns the [bluemap] image containing the
   red components of pixels in [pxm]. Similarly for green and blue components
   for [bluemap_of_green_channel] and [bluemap_of_blue_channel]. *)

val bluemap_of_hue_channel : pixmap -> bluemap;;
val bluemap_of_saturation_channel : pixmap -> bluemap;;
val bluemap_of_brightness_channel : pixmap -> bluemap;;
 (** [bluemap_of_hue_channel pxm] returns the [bluemap] image containing the
   hue components of pixels in [pxm]. Similarly for saturation and brightness
   components for [bluemap_of_saturation_channel] and
   [bluemap_of_brightness_channel]. *)

val pixmap_of_rgb_bluemaps : bluemap -> bluemap -> bluemap -> pixmap;;
 (** [pixmap_of_rgb_bluemaps rgm ggm bgm] returns the pixmap [pxm] such that
   [bluemap_of_red_channel pxm = rgm], [bluemap_of_green_channel pxm = ggm],
   and [bluemap_of_blue_channel pxm = bgm]. *)

val pixmap_of_hsb_bluemaps : bluemap -> bluemap -> bluemap -> pixmap;;
 (** [pixmap_of_rgb_bluemaps hgm sgm bgm] returns the pixmap [pxm] such that
   [bluemap_of_hue_channel pxm = hgm], [bluemap_of_saturation_channel pxm = sgm],
   and [bluemap_of_brightness_channel pxm = sgm]. *)

val gray_pixmap_of_bluemap : bluemap -> pixmap;;
 (** This returns the [pixmap] corresponding to a [bluemap], when the [bluemap]
   is considered as a greymap. *)

val red_pixmap_of_bluemap : bluemap -> red_pixmap;;
val green_pixmap_of_bluemap : bluemap -> green_pixmap;;
val blue_pixmap_of_bluemap : bluemap -> blue_pixmap;;
 (** The functions that returns the [pixmap] corresponding to a [bluemap], when
   the [bluemap] is considered as the [bluemap] for the [red], [green], or
   [blue] color. *)

val red_pixmap_of_red_channel : pixmap -> red_pixmap;;
val green_pixmap_of_green_channel : pixmap -> green_pixmap;;
val blue_pixmap_of_blue_channel : pixmap -> blue_pixmap;;
 (** The functions that returns the [pixmap] corresponding to a [red], [green],
   or [blue] channel of a [pixmap]. *)

val rgb_pixmap_of_rgb_channels :
  red_pixmap -> green_pixmap -> blue_pixmap -> pixmap;;
 (** The functions that returns the [pixmap] corresponding to the image having
   the corresponding [red], [green], and [blue] channels. *)

val map : (Graphics.color -> Graphics.color) -> pixmap -> pixmap;;
 (** [map f pixmap] maps function [f] on each pixel of the given pixmap
   and returns the resulting pixmap. *)

val set_pixmap_pixels :
 (Graphics.color -> Graphics.color) -> pixmap -> unit;;
 (** [set_pixmap_pixels f pxm] sets each pixel [p] of pixmap [pxm] with the
   value [f p]. *)

(** Transparency. *)

val set_pixels_transp : (Graphics.color -> bool) -> pixmap -> unit;;

 (** Set to transparent the pixels of a pixmap whose color verifies a given
   predicate. *)

val set_background_transp : pixmap -> unit;;
 (** Set to transparent the pixels of a pixmap whose color is the background
   color of the graphics window. In some sense this function embosses a pixmap
   image. *)

val generic_image_of_pixmap : pixmap -> Generic_image.image;;
 (** Convert a pixmap to a [Generic_image.image] image. *)
val pixmap_of_generic_image : Generic_image.image -> pixmap;;
 (** Convert a [Generic_image.image] value to an array of colors. *)

val image_of_generic_image : Generic_image.image -> Graphics.image;;
 (** Convert a [Generic_image] value to an image for [Graphics]. *)
val generic_image_of_image : Graphics.image -> Generic_image.image;;
 (** The converse convertion. *)

val image_of_pixmap : pixmap -> Graphics.image;;
 (** Another name for [Graphics.make_image]. *)
val pixmap_of_image : Graphics.image -> pixmap;;
 (** Another name for [Graphics.dump_image]. *)
